package dev.shadowsoffire.apotheosis.adventure.compat;

import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.cutting.GemCuttingMenu;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import java.util.List;
import java.util.Arrays;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.cutting.GemCuttingBlock;
import net.minecraft.network.chat.Component;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.ingredients.IIngredientType;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import net.minecraft.world.level.ItemLike;
import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.helpers.IGuiHelper;
import mezz.jei.api.gui.drawable.IDrawable;
import net.minecraft.resources.ResourceLocation;
import mezz.jei.api.recipe.category.IRecipeCategory;

public class GemCuttingCategory implements IRecipeCategory<GemCuttingRecipe>
{
    public static final ResourceLocation TEXTURES;
    private final IDrawable background;
    private final IDrawable icon;
    
    public GemCuttingCategory(final IGuiHelper guiHelper) {
        this.background = (IDrawable)guiHelper.drawableBuilder(GemCuttingCategory.TEXTURES, 0, 0, 148, 78).addPadding(0, 0, 0, 0).build();
        this.icon = guiHelper.createDrawableIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)new ItemStack((ItemLike)Adventure.Blocks.GEM_CUTTING_TABLE.get()));
    }
    
    public RecipeType<GemCuttingRecipe> getRecipeType() {
        return AdventureJEIPlugin.GEM_CUTTING;
    }
    
    public Component getTitle() {
        return GemCuttingBlock.NAME;
    }
    
    public IDrawable getBackground() {
        return this.background;
    }
    
    public IDrawable getIcon() {
        return this.icon;
    }
    
    public void setRecipe(final IRecipeLayoutBuilder builder, final GemCuttingRecipe recipe, final IFocusGroup focuses) {
        builder.addSlot(RecipeIngredientRole.INPUT, 48, 37).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)recipe.gem);
        builder.addSlot(RecipeIngredientRole.INPUT, 48, 4).addIngredients((IIngredientType)VanillaTypes.ITEM_STACK, (List)Arrays.asList(recipe.materials));
        builder.addSlot(RecipeIngredientRole.INPUT, 19, 56).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)recipe.gem);
        builder.addSlot(RecipeIngredientRole.INPUT, 76, 56).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)recipe.dust);
        builder.addSlot(RecipeIngredientRole.OUTPUT, 117, 35).addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)recipe.out);
    }
    
    static {
        TEXTURES = new ResourceLocation("apotheosis", "textures/gui/gem_cutting_jei.png");
    }
    
    public static class GemCuttingRecipe
    {
        protected final ItemStack out;
        protected final ItemStack gem;
        protected final ItemStack dust;
        protected final ItemStack[] materials;
        
        public GemCuttingRecipe(final Gem gem, final LootRarity rarity) {
            this.out = GemRegistry.createGemStack(gem, rarity.next());
            this.gem = GemRegistry.createGemStack(gem, rarity);
            this.dust = new ItemStack((ItemLike)Adventure.Items.GEM_DUST.get(), GemCuttingMenu.getDustCost(rarity));
            final LootRarity min = (LootRarity)RarityRegistry.getMinRarity().get();
            if (rarity == min) {
                (this.materials = new ItemStack[2])[0] = new ItemStack((ItemLike)rarity.getMaterial(), 3);
                this.materials[1] = new ItemStack((ItemLike)rarity.next().getMaterial(), 1);
            }
            else if ("ancient".equals(RarityRegistry.INSTANCE.getKey((CodecProvider)rarity.next()).m_135815_())) {
                (this.materials = new ItemStack[2])[0] = new ItemStack((ItemLike)rarity.prev().getMaterial(), 9);
                this.materials[1] = new ItemStack((ItemLike)rarity.getMaterial(), 3);
            }
            else {
                (this.materials = new ItemStack[3])[0] = new ItemStack((ItemLike)rarity.prev().getMaterial(), 9);
                this.materials[1] = new ItemStack((ItemLike)rarity.getMaterial(), 3);
                this.materials[2] = new ItemStack((ItemLike)rarity.next().getMaterial(), 1);
            }
        }
    }
}
